

#ifndef __MD5_H
#define __MD5_H


#ifdef __cplusplus
extern "C" {
#endif

// (calling convention and export model)
#define MD5_API


// size of an MD5 digest
#define MD5_DIGESTSIZE  16


// MD5 context

typedef struct
{
  unsigned int state[4];  // state (ABCD)
  unsigned int count[2];  // number of bits, modulo 2^64 (lsb first)
  unsigned char buffer[64]; // input buffer
}
MD5CTX, *PMD5CTX;



/*
 * initialization, begins an MD5 operation, writing a new context
 * <- pointer to new MD5 context
 */
PMD5CTX MD5_API 
    MD5_Create ();


/*
 * same initialization, but statically
 * -> pointer to MD5 context
 */
void MD5_API 
    MD5_Initialize (PMD5CTX);


/*
 * resets an MD5 context
 * -> context to rest
 */
void MD5_API 
    MD5_Reset (PMD5CTX);


/*
 * releases an MD5 context
 * -> pointer to MD5 context to release
 */
void MD5_API 
    MD5_Destroy (PMD5CTX);


/*
 * block update operation, continues an MD5 message-digest operation,
 * processing another message block, and updating the context
 * -> pointer to MD5 context
 * -> pointer to input buffer, which is treated as a unsigned char buffer
 * -> number of unsigned chars to scramble
 */
void MD5_API 
    MD5_Update (PMD5CTX, const void*, unsigned int);



/*
 * MD5 finalization, ends an MD5 message-digest operation,
 * writing the message digest and clearing the context
 * -> buffer where to copy the digest's unsigned chars
 * -> pointer to MD5 context
 */
void MD5_API 
    MD5_Final (unsigned char[MD5_DIGESTSIZE], PMD5CTX);


/*
 * selftest
 * <- 1: selftest succeded / 0: error
 */
int MD5_API 
    MD5_SelfTest();


#ifdef __cplusplus
}
#endif


#endif
